# frozen_string_literal: true

module Referator
  class Config
    class Categories
      attr_reader :config, :names

      def initialize(config)
        self.config = config
        @names = []
      end

      def freeze
        @names.freeze
        super
      end

      def register(name)
        name = name.to_sym if name.instance_of? String
        Referator.validate_name! name
        raise 'Already exists' if names.index name

        names << name
        nil
      end

      def exists!(name)
        names.index name or raise 'Unknown category'
        nil
      end

    private

      def config=(config)
        unless config.instance_of? Config
          raise TypeError, "Expected #{Config}, got #{config.class}"
        end

        @config = config
      end
    end
  end
end
