# frozen_string_literal: true

module Referator
  class Config
    class Repo
      attr_reader :config

      def initialize(config)
        self.config = config
        @references = {}
      end

      def freeze
        @references.freeze
        super
      end

      def register_ref(**kwargs)
        reference = Reference.new(**kwargs)
        key = "#{reference.category}-#{reference.id}".freeze
        raise 'Reference already exists' if @references.key? key

        @references[key] = reference
        nil
      end

      def [](category, id)
        category = category.to_sym if category.instance_of? String
        config.categories.exists! category
        key = "#{category}-#{id}".freeze
        @references[key].tap do |reference|
          raise 'Invalid reference' if reference.nil?
        end
      end

    private

      def config=(config)
        unless config.instance_of? Config
          raise TypeError, "Expected #{Config}, got #{config.class}"
        end

        @config = config
      end
    end
  end
end
