# frozen_string_literal: true

module Referator
  class Config
    class Scripts
      attr_reader :config

      def initialize(config)
        self.config = config
        @scripts = {}
      end

      def freeze
        @scripts.freeze
        super
      end

      def register(name:, workdir: nil, vars: [], **kwargs)
        name = String(name).to_sym
        Referator.validate_name! name
        raise 'Already exists' if @scripts.key? name

        script = Script.new(
          **kwargs,
          workdir: workdir || config.workdir,
          vars: vars.map { |var| var.instance_of?(String) ? var.to_sym : var },
        )

        @scripts[name] = script
        nil
      end

      def vars!(name, vars)
        unless vars.instance_of? Array
          raise TypeError, "Expected #{Array}, got #{vars.class}"
        end

        Referator.validate_name! name
        vars.each { |var| Referator.validate_script_var! var }

        script = @scripts[name] or raise 'Invalid script'

        raise 'Invalid vars' unless script.vars.sort == vars.sort
      end

      def run(name, **vars)
        vars! name, vars.keys
        @scripts[name].call(**vars)
      end

    private

      def config=(config)
        unless config.instance_of? Config
          raise TypeError, "Expected #{Config}, got #{config.class}"
        end

        @config = config
      end
    end
  end
end
