# frozen_string_literal: true

module Referator
  class Reference
    attr_reader :category, :id, :slug, :data

    def initialize(category:, id:, slug:, **kwargs)
      self.category = category
      self.id       = id
      self.slug     = slug
      self.data     = kwargs
    end

    def to_h
      @to_h ||= data.merge(category:, id:, slug:, anchor:, fragment:).freeze
    end

    def anchor
      @anchor ||= "#{category}-#{slug}".freeze
    end

    def fragment
      @fragment ||= "##{anchor}".freeze
    end

  private

    def category=(category)
      @category = Referator.validate_name! String(category).to_sym
    end

    def id=(id)
      @id = String(id).freeze
    end

    def slug=(slug)
      @slug = Referator.validate_slug! String slug
    end

    def data=(data)
      @data = Hash(data).transform_keys { |key| String(key).to_sym }.freeze
    end
  end
end
